function [features, targets, UW] = PCA(features, targets, dimension, region)

%Reshape the data points using the principal component analysis
%Inputs:
%	train_features	- Input features
%	train_targets	- Input targets
%	dimension		- Number of dimensions for the output data points
%	region			- Decision region vector: [-x x -y y number_of_points]
%
%Outputs
%	features			- New features
%	targets			- New targets
%	UW					- Reshape martix
%  mf					- Reshape means

[r,c] = size(features);

if (r < dimension),
   disp('Required dimension is larger than the data dimension.')
   disp(['Will use dimension ' num2str(r)])
   dimension = r;
end

%Calculate cov matrix and the PCA matrixes
Q			= (features * features')/c;
[V, D]	                = eig(Q);
W			= V(:,r-dimension+1:r)';
U			= Q*W'*inv(W*Q*W');

%Calculate new features
UW			= U*W;
features = UW*features;
