% Computes the class conditional errors and the overall error rate
%
% usage:
%    error = computeError(classes, trueLabels, estLabels)
%
% where:
%     classes    = class labels (ith class has label classes(i)), to deal with 
%                   arbitrary class labels
%     origLabels = the true labels
%     estLabels  = labels produced by the classifier
%
%     error      = the error rates; error(1:Nclasses) are the
%                  class-conditional error rates, error(Nclasses+1) 
%                  is the overall error rate

function error = computeError(classes, origLabels, estLabels)

[a b] = size(origLabels);
if ( a ==1 & b > 1),  origLabels = origLabels'; end
[a b] = size(estLabels);
if ( a == 1 & b > 1),  estLabels = estLabels'; end


Nclasses = length(classes);
error    = zeros(Nclasses+1,1);

mistakes = (estLabels ~= origLabels);
% compute the class-conditional error rate, and the overall error rate
%
for cl = 1:Nclasses,
  samWithCurrLabel  = (origLabels == classes(cl));
  curr_errors       = sum(mistakes.*samWithCurrLabel);
  if (curr_errors > 0)
    error(cl)         = curr_errors/sum(samWithCurrLabel);
  else
    error(cl) = 0;
  end
  error(Nclasses+1) = error(Nclasses+1) + curr_errors;
end
error(Nclasses+1) = error(Nclasses+1)/length(origLabels);
