% Classification GUI and toolbox
% Version 1.0
% 
% Modified by Vittorio Castelli, 2002 (vittorio@ee.columbia.edu)
%
% The topmost box in the user interface selects between 
%  "Original Framework" and "Framework for class"
%
% The Framework for the class has the following characteristics:
%  - it works with any number of features, any number of labels
%  - it does not produce plottable decision regions (because Ndim can be > 2)
%  - it does not support all the original algorithms
%  - it supports 2 additional algorithms (maybe more in the future)
%      (weighted KNN, and K-l NN)
%  - it prints the conditional and overall probabilities of error to the
%    standard output.
%  - the "Parzen windows" method uses the kernel functions described in class
%  - it uses an "advanced" toolbox for SVM (SLOW!)
%  
%  The  framework for the class does not support all the preprocessing
%   methods, in particular those that were written specifically for the 2-class problem,
%  All the classifiers written for the class framework have the suffix _VC.m
%  or _VCcore.m
%
%
% GUI start commands
%
%		classifier		- Start the classification GUI
%		enter_distributions     - Starts the parameter input screen (used by classifier)
%		multialgorithms		- Start the algorithm comparison screen 
%		                          (original framework only)
%
% Preprocessing methods
%
%  Currently implemented for 2-class problem
%	Deterministic_annealing - Compute k features which typify the data using the DA algorithm
%	Fuzzy_k_means		- Compute k means for the data using the fuzzy_k_means algorithm
%	k_means			- Compute k means for the data using the k_means algorithm
%	ADDC			- Compute k clusters for the data using the agglomerative clustering method
%	LVQ1			- Linear vector quantization with one neighbor
%	LVQ3			- Linear vector quantization 3 algorithm
%	DSLVQ			- Distinction sensitive linear vector quantization 
%  Implemented for any class problem
%	PCA			- Principal component analysis
%	Fisher linear disc	- Fisher linear discriminant
%
% Classification Algorithms
%  (* = not in new set,  x = not in old set, 
%   + = for more than 2 classes, creates 1 classifier per each label pair, followed by majority vote)
%
% Parametric classification algorithms
%
%	EM			- Expectation maximization algorithm
%	  p_single		- Used by EM algorithm
% *     RDA			- Regularized descriminant analysis (Friedman shrinkage algorithm)
% +	LS			- Least squares algorithm
%	ML			- Maximum likelihood algorithm
%	ML_diag	    		- Maximum likelihood with diagonal covariance matrices
% +	Perceptron		- Sigle perceptron algorithm
% +	Pocket			- Pocket algorithm
%	  longest_run		- Used by the Pocket algorithm 
% *     Stumps                  - Simple stump classifier
%
% Non-parametric classification algorithms 
%
% *     Ada_Boost           - Ada Boost algorithm
%	Knn_Rule	    - Used by store_grabbag
% *     Local_Polynomial    - Local polynomial fitting
%	  loglikelihood	    - Used by local_polynomial
% *     LocBoost            - Local boosting
%         LcoBoostFunctions   - Used by LocBoost
%	Nearest_Neighbor    - Nearest neighbor algorithm
%	Parzen		    - Parzen window algorithm
%	PNN		    - Probabilistic neural network
%	RCE		    - Reduced coulomb energy algorithm
% *	Store_Grabbag	    - An improvement on the nearest neighbor algorithm
% +     SVM                 - Support vector machines
% *     Voted perceptron    - Voted perceptron algorithm.
% x     weightedKNNRule     - weighted k nearest neighbor
%
%
% NOTE: the SVM algorithm now uses the SVM toolbox for Matlab, v2.51, please
%       look at the ReadmeSVM.txt and LicenseSVM.txt for more information.
% 
% Error estimation
%
%		calculate_error		- Calculates the classification error given a decision surface
%		classification_error    - Used by claculate_error
%		decision_region		- Builds a decision region for multi-Gaussian distributions
%
% GUI housekeeping functions
%
%	calculate_region	- Finds the data scatter region
%	change_class		- Used by enter_distributions
%	change_gaussian		- Used by enter_distributions
%	change_parameter	- Used by enter_distributions
%	classifier_commands	- Classifier screen housekeeping commands
%	find_classes		- Find which classes exist in a data set
%	generate_data_set	- Generate a data set given Gaussian parameters
%	load_file		- Load data files
%	make_a_draw		- Randomly find indices from a data set
%	MoveAlgorithm		- Used by multialgorithms
%       plot_process            - Plot partition centers during the algorithm execution
%	plot_scatter		- Make a scatter plot of a data set
%	start_classify		- Main function used by classifier
%	read_algorithms	        - Reads an algorithm file into a data structure
%	start_multi_classification - main function used by multialgorithms
%       SVM_params_window       - The SVM parameter input form
%       SVM_params_window_commands - The command for the SVM paramter info screen
%	voronoi_regions		- Plot Voronoi regions
%	write_svm_data		- Write SVM data in the SVMlight format
%
% Data sets (Ending _data means that the file contains features, 
%                   _params means that the file contains the distribution parameters)
%
%		clouds				- A data set composed of four Gaussians 
%		seperable			- A linearly seperable data set
%		synthetic			- A data set built according to a distribution
%
%
%____________________________________________________________________________________
%  Elad Yom-Tov (elad@ieee.org) and Hilit Serby
%  Technion - Israel Institute of Technology
%  Haifa, Israel
